package org.nl.system.service.dept.dao.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Param;
import org.nl.system.service.dept.dao.SysDept;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 部门 Mapper 接口
 * </p>
 *
 * @author generator
 * @since 2022-12-15
 */
public interface SysDeptMapper extends BaseMapper<SysDept> {

    /**
     * 保存依赖关系
     * @param UserId
     * @param deptId
     */
    void saveDeptRelation(@Param("user") String UserId,@Param("depts") Collection<String> deptId);
    void delDeptRelation(@Param("user") String UserId);

    List<Map> getDeptRelation(@Param("deptIds") Collection<String> deptIds);

    /**
     * 跟新sub_count字段
     * @param deptId
     * @return
     */
    int updateSubCount(String deptId);

    /**
     * 返回字符串列表 split = ,
     * @param pid
     * @return
     */
    String findAllChild(String pid);
}
